@@ -11,6 +11,43 @@ from scipy import signal
 import numpy as np
 import os
 
+def get_power_spectrum(time_quantity, constant):
+    Tkw = np.fft.fft(time_quantity, axis=0)
+    S = (np.sum(np.conjugate(Tkw)*Tkw, axis=1)).real*constant
+    return S
+
+def get_power_spectrum_proj(time_quantity, constant):
+    Tkw = np.fft.fft(time_quantity, axis=0)
+    S = (np.conjugate(Tkw)*Tkw).real*constant
+    return S
+
+def get_power_spectrum2(time_quantity, dt_ps, constant, Nb):
+    Num_timesteps = len(time_quantity)
+    tau = int(Num_timesteps/Nb)
+    S = np.zeros(tau)
+    for b in range(Nb):
+        tkt = time_quantity[b*tau:(b+1)*tau]
+        tw = np.fft.fft(tkt, axis=0)
+        S = S + (np.sum(np.conjugate(tw)*tw, axis=1)).real*constant/tau*dt_ps
+    freq = np.fft.fftfreq(tau, dt_ps)
+    return tau, freq, S/Nb
+
+def get_power_spectrum_proj2(time_quantity, dt_ps, constant, Nb):
+    Num_timesteps = len(time_quantity)
+    ndofs = len(time_quantity[0,:])
+    tau = int(Num_timesteps/Nb)
+    S = np.zeros((tau, ndofs))
+    for b in range(Nb):
+        tkt = time_quantity[b*tau:(b+1)*tau]
+        tw = np.fft.fft(tkt, axis=0)
+        S = S + (np.conjugate(tw)*tw).real*constant/tau*dt_ps
+    freq = np.fft.fftfreq(tau, dt_ps)
+    return tau, freq, S/Nb
+
+
+
+
+
 
 def get_commensurate_kpath(file, N1,N2,N3, N1_FC,N2_FC,N3_FC, kinput_scaled, labels):
     # =============================================================================
@@ -241,8 +278,7 @@ def get_eigvals_eigvec(D):
     eigvals, eigvecs = eigvals[idx], eigvecs[:,idx]
     omegas = np.sqrt(eigvals, dtype=complex)/(2*np.pi)
     omegas = omegas.real - omegas.imag
-    eigvecs_ortho, R = np.linalg.qr(eigvecs)
-    return eigvals, eigvecs_ortho, omegas
+    return eigvals, eigvecs, omegas
 
 def get_eigvals(D):
     eigvals = np.linalg.eigvalsh(D)
@@ -469,7 +505,7 @@ def get_real_branches(freqs, eigvecs, masses, ks):
             
         this_eigvecs_reshaped = this_eigvecs_norm[:, new_indexes]
         
-        dot = np.abs(np.dot(this_eigvecs_reshaped.T, next_eigvecs_norm)) #* (np.dot(next_eigvecs_norm.T, this_eigvecs_reshaped)).conj()
+        dot = np.abs(np.dot(this_eigvecs_reshaped.conj().T, next_eigvecs_norm)) #* (np.dot(next_eigvecs_norm.T, this_eigvecs_reshaped)).conj()
         for i in range(num_branches):
             row = dot[i,:]
             max_index, max_row = int(np.argwhere(row == max(row))), max(row)
@@ -701,7 +737,7 @@ def get_K_renorm_old(tot_atoms_uc, N1, N2, N3, H, R0, kpoints, Ds):
     N1N2N3_k = len(kpoints)
     N = N1N2N3 * tot_atoms_uc
     R_mat = R_matrix(N1,N2,N3,tot_atoms_uc, R0, H)
-    
+
     K_renorm = np.zeros((tot_atoms_uc*3, N*3), dtype=complex)
     for i in range(0,tot_atoms_uc*3,3):
         atom_ref_i = int(i/3)
@@ -713,6 +749,24 @@ def get_K_renorm_old(tot_atoms_uc, N1, N2, N3, H, R0, kpoints, Ds):
             K_renorm[i:i+3,j*3:j*3+3] = KR[i:i+3,atom_ref_j*3:atom_ref_j*3+3]
     return  K_renorm.real/(0.964*10**(4))
 
+
+def sum_Dqs_old(R,kpoints,Ds, N1N2N3):
+    """
+    old sum Dqs, without exploiting irr ks
+    """
+    kpoints2 = -kpoints
+    # kpoints = np.vstack((kpoints,kpoints2))
+    Ds2 = np.conjugate(Ds[:,:,:])
+    # Ds = np.concatenate((Ds, Ds2), axis=0)
+    kdotR = np.dot(kpoints,R)
+
+    cs = np.exp(1j*kdotR).reshape(N1N2N3,1,1)
+
+
+    A = 1/N1N2N3*np.multiply(cs,Ds)
+    Kr = np.sum(A, axis=0)
+    return Kr
+
 def get_K_renorm(tot_atoms_uc, N1, N2, N3, H, R0, irr_ks, permutations_types, Ds):
     N1N2N3_k = len(Ds[:,0,0])
     N1N2N3 = N1*N2*N3
@@ -732,25 +786,6 @@ def get_K_renorm(tot_atoms_uc, N1, N2, N3, H, R0, irr_ks, permutations_types, Ds
     print('Maximum imaginary part of renorm FORCE CONSTANTS:', np.max(K_renorm.imag), '[ev/B]')
     return  K_renorm.real
 
-
-
-
-def sum_Dqs_old(R,kpoints,Ds, N1N2N3):
-    """
-    old sum Dqs, without exploiting irr ks
-    """
-    kpoints2 = -kpoints
-    # kpoints = np.vstack((kpoints,kpoints2))
-    Ds2 = np.conjugate(Ds[:,:,:])
-    # Ds = np.concatenate((Ds, Ds2), axis=0)
-    kdotR = np.dot(kpoints,R)
-    
-    cs = np.exp(1j*kdotR).reshape(N1N2N3,1,1)
-
-    A = 1/N1N2N3*np.multiply(cs,Ds)
-    Kr = np.sum(A, axis=0)
-    return Kr
-
 def sum_Dqs(R,irr_kpoints, permutations_types, Ds, N1N2N3):
     N1N2N3_irr = len(irr_kpoints)
     permutations = np.array([[[0,1,0],[1,0,0],[0,0,1]], 
@@ -882,6 +917,24 @@ def get_Scells(H, R0):
         Scells[c] = R0 + displ[c]     
     return Scells
 
+def R_matrix2(N1,N2,N3,tot_atoms_uc,R0):
+    """
+    returns the R matrix
+    """
+    N1N2N3 = N1 * N2 * N3
+    N = len(R0[:,0])
+    
+    R_matrix = np.zeros((tot_atoms_uc*3,N*3))
+    count = 0
+    for i in range(tot_atoms_uc):
+        atom_origin = R0[i*N1N2N3]
+        row = (R0[:,:]-atom_origin).flatten()
+        repeated_row = np.tile(row,(3,1))
+        R_matrix[count:count+3,:] = repeated_row
+        count =  count + 3
+        
+    return R_matrix#*0.529177249
+
 def R_matrix(N1,N2,N3,tot_atoms_uc,R0, H):
     """
     returns the R matrix
@@ -930,13 +983,14 @@ def get_D(k, K,N1,N2,N3,R0,masses,tot_atoms_uc, H):
     
     D = np.zeros((tot_atoms_uc*3,tot_atoms_uc*3),dtype=complex)
     for i in range(tot_atoms_uc*3):
+        # for j in range(1):
         for j,h in zip(range(tot_atoms_uc*3),np.repeat(range(0,N),3)):
 #            print(i,j,h)
             mass_coeff = complex(1/(masses[i]*masses[j])**(0.5))
             exp = 0*1j
-#            h = 1
+            # h = 0
             for l in range(j+h*3*(N1N2N3-1),j+h*3*(N1N2N3-1)+(N1N2N3)*3,3):
-#                print(i,l,np.round(np.exp(1j*kdotr[i,l]),2),K[i,l])
+                # print(i,l,np.round(np.exp(1j*kdotr[i,l]),2),K[i,l])
                 exp = exp + K[i,l]*np.exp(1j*kdotr[i,l])
 #            print()
             D[i,j] = mass_coeff*exp
